@@ -4,29 +4,23 @@ import platform
 import click
 import re
 import yaml
-import json
-import csv
-import subprocess
 import glob
-from pathlib import Path
 from stdiomask import getpass
-from .constants import *
+from .cli_constants import *
 from .exceptions import DSOException
-from .config import Config
+from .config import Configs
 from .logger import Logger, log_levels
 from .stages import Stages
 from .parameters import Parameters
 from .secrets import Secrets
 from .templates import Templates
-from .packages import Packages
-from .releases import Releases
 from .click_extend import *
 from click_params import RangeParamType
 from .cli_utils import *
 from .file_utils import *
-from .version import dso_version
 from functools import reduce
 from .pager import Pager
+from .version import __version__
 
 
 
@@ -34,7 +28,7 @@ from .pager import Pager
 
 @click.group(context_settings=DEFAULT_CLICK_CONTEXT)
 def cli():
-    """DevSecOps CLI"""
+    """DevSecOps Tool CLI"""
     pass
 
 ###--------------------------------------------------------------------------------------------
@@ -118,7 +112,7 @@ def version():
     """
     Display version details.
     """
-    click.echo(f"DSO CLI: {dso_version}\nPython: {platform.sys.version}")
+    click.echo(f"DSO CLI: {__version__}\nPython: {platform.sys.version}")
 
 
 ###--------------------------------------------------------------------------------------------
@@ -160,9 +154,9 @@ def add_parameter(key, key_option, value, value_option, stage, input, format, ve
         ### no input file
         else:
             validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-            key = validate_only_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+            key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
             validate_not_all_provided([key, key_option], ["VALUE", "'-v' / '--value'"])
-            value = validate_only_one_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
+            value = validate_at_least_one_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
             parameters.append({'Key': key, 'Value': value})
 
     success = []
@@ -170,7 +164,7 @@ def add_parameter(key, key_option, value, value_option, stage, input, format, ve
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         failed = [x['Key'] for x in parameters]
         for param in parameters:
@@ -216,7 +210,7 @@ def list_parameter(stage, uninherited, query_all, query, format, verbosity, conf
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         result = Parameters.list(stage, uninherited)
         output = format_data(result, query, format)
         if output: Pager.page(output)
@@ -257,7 +251,7 @@ def get_parameter(key, key_option, stage, revision, history, query_all, query, f
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         if history:
             result = Parameters.history(stage, key)
         else:
@@ -299,7 +293,7 @@ def delete_parameter(key, key_option, input, format, stage, verbosity, config, w
         ### no input file
         else:
             validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-            key = validate_only_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+            key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
             parameters.append({'Key': key})
 
     success = []
@@ -307,7 +301,7 @@ def delete_parameter(key, key_option, input, format, stage, verbosity, config, w
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         failed = [x['Key'] for x in parameters]
         for parameter in parameters:
@@ -368,7 +362,7 @@ def add_secret(key, key_option, stage, input, format, verbosity, config, working
         ### no input file
         else:
             validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-            key = validate_only_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+            key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
             value = getpass("Enter secret value: ")
             value2 = getpass("Verify secret value: ")
             if not value == value2:
@@ -381,7 +375,7 @@ def add_secret(key, key_option, stage, input, format, verbosity, config, working
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         failed = [x['Key'] for x in secrets]
         for secret in secrets:
@@ -428,7 +422,7 @@ def list_secret(stage, uninherited, decrypt, query_all, query, format, verbosity
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         result = Secrets.list(stage, uninherited, decrypt)
         output = format_data(result, query, format)
         if output: Pager.page(output)
@@ -469,7 +463,7 @@ def get_secret(key, key_option, stage, revision, history, query_all, query, form
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         if history:
             result = Secrets.history(stage, key)
         else:
@@ -512,7 +506,7 @@ def delete_secret(key, key_option, input, format, stage, verbosity, config, work
         ### no input file
         else:
             validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
-            key = validate_only_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+            key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
             secrets.append({'Key': key})
 
     success = []
@@ -520,7 +514,7 @@ def delete_secret(key, key_option, input, format, stage, verbosity, config, work
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         failed = [x['Key'] for x in secrets]
         for secret in secrets:
@@ -566,7 +560,7 @@ def list_template(uninherited, query_all, query, format, verbosity, config, work
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         result = Templates.list(uninherited)
         output = format_data(result, query, format)
         if output: Pager.page(output)
@@ -597,7 +591,7 @@ def get_template(key, key_option, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         print(Templates.get(key), flush=True)
     
     except DSOException as e:
@@ -687,7 +681,7 @@ def add_template(key, key_option, render_path, input, recursive, verbosity, conf
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         failed = [x['Key'] for x in templates]
         for template in templates:
@@ -735,7 +729,8 @@ def delete_template(key, key_option, input, format, verbosity, config, working_d
             templates = read_data(input, 'Templates', ['Key'], format)
         ### no input file
         else:
-            key = validate_only_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+            validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+            key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
             templates.append({'Key': key})
 
     success = []
@@ -743,7 +738,7 @@ def delete_template(key, key_option, input, format, verbosity, config, working_d
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         failed = [x['Key'] for x in templates]
         for template in templates:
@@ -783,7 +778,7 @@ def render_template(stage, limit, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         success = Templates.render(stage, limit)
         result = {'Success': success, 'Failure': []}
@@ -821,7 +816,7 @@ def list_package(stage, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -855,7 +850,7 @@ def download_package(stage, package, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -889,7 +884,7 @@ def create_package(stage, description, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -923,7 +918,7 @@ def delete_package(stage, package, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -955,7 +950,7 @@ def list_release(stage, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -989,7 +984,7 @@ def download_release(stage, release, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -1024,7 +1019,7 @@ def create_release(stage, description, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -1059,7 +1054,7 @@ def delete_release(stage, release, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -1093,7 +1088,7 @@ def deploy_release(stage, release, format, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
         raise NotImplementedError("Not implemented.")
     
@@ -1110,26 +1105,30 @@ def deploy_release(stage, release, format, verbosity, config, working_dir):
 @config.command('get', context_settings=DEFAULT_CLICK_CONTEXT, short_help=f"{CLI_COMMANDS_SHORT_HELP['config']['get']}")
 @command_doc(CLI_COMMANDS_HELP['config']['get'])
 @click.argument('key', required=False)
+@click.option('-k', '--key', 'key_option', metavar='<value>', required=False, help=f"{CLI_PARAMETERS_HELP['config']['key']}")
 @click.option('-l','--local', is_flag=True, default=False, help=f"{CLI_PARAMETERS_HELP['config']['local']}")
 @click.option('-g','--global', 'global_', is_flag=True, default=False, help=f"{CLI_PARAMETERS_HELP['config']['global']}")
 @click.option('-b', '--verbosity', metavar='<number>', required=False, type=RangeParamType(click.INT, minimum=0, maximum=5), default='2', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['verbosity']}")
 @click.option('--config', metavar='<key>=<value>,...', required=False, help=f"{CLI_PARAMETERS_HELP['common']['config']}")
 @click.option('-w','--working-dir', metavar='<path>', type=click.Path(exists=True, file_okay=False), required=False, default='.', show_default=True, help=f"{CLI_PARAMETERS_HELP['common']['working_dir']}")
-def get_config(key, local, global_, verbosity, config, working_dir):
+def get_config(key, key_option, local, global_, verbosity, config, working_dir):
 
     scope = None
 
     def validate_command_usage():
-        nonlocal scope
+        nonlocal scope, key
+        validate_not_all_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+        key = validate_at_least_one_provided([key, key_option], ["KEY", "'-k' / '--key'"])
+
         validate_not_all_provided([local, global_], ["'-l' / '--local'", "'-g' / '--global'"])
         scope = 'local' if local else 'global' if global_ else ''
 
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
 
-        print(yaml.dump(Config.get(key, scope), sort_keys=False, indent=2), flush=True)
+        print(yaml.dump(Configs.get(key, scope), sort_keys=False, indent=2), flush=True)
 
     except DSOException as e:
         Logger.error(e.message)
@@ -1167,14 +1166,14 @@ def set_config(key, key_option, value, value_option, global_, input, verbosity,
             except:
                 raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format('yaml'))
         else:
-            validate_not_all_provided([key, key_option], ["VALUE", "'-v' / '--value'"])
-            value = validate_only_one_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
+            validate_not_all_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
+            value = validate_at_least_one_provided([value, value_option], ["VALUE", "'-v' / '--value'"])
 
     try:
         Logger.set_verbosity(verbosity)
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         validate_command_usage()
-        Config.set(key, value, global_)
+        Configs.set(key, value, global_)
 
     except DSOException as e:
         Logger.error(e.message)
@@ -1203,9 +1202,9 @@ def delete_config(key, key_option, global_, verbosity, config, working_dir):
 
     try:
         Logger.set_verbosity(verbosity)
-        Config.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.load(working_dir if working_dir else os.getcwd(), config)
         validate_command_usage()
-        Config.delete(key, global_)
+        Configs.delete(key, global_)
 
     except DSOException as e:
         Logger.error(e.message)
@@ -1231,7 +1230,7 @@ def delete_config(key, key_option, global_, verbosity, config, working_dir):
 
 #     try:
 #         Logger.set_verbosity(verbosity)
-#         Config.load(working_dir if working_dir else os.getcwd(), config)
+#         Configs.load(working_dir if working_dir else os.getcwd(), config)
 #         validate_command_usage()
 
 
@@ -1271,8 +1270,8 @@ def init_config(setup, local, input, verbosity, config, working_dir):
     try:
         Logger.set_verbosity(verbosity)
         validate_command_usage()
-        # Config.load(working_dir if working_dir else os.getcwd(), config)
-        Config.init(working_dir, init_config, config, local)
+        # Configs.load(working_dir if working_dir else os.getcwd(), config)
+        Configs.init(working_dir, init_config, config, local)
 
     except DSOException as e:
         Logger.error(e.message)
@@ -1284,7 +1283,8 @@ def init_config(setup, local, input, verbosity, config, working_dir):
 
 ###--------------------------------------------------------------------------------------------
 
+modify_click_usage_error()
+
 if __name__ == '__main__':
     cli()
     
-modify_click_usage_error()
