@@ -1,6 +1,6 @@
 from .logger import Logger
-from .config import Config
-from .providers import StoreProvider, ProviderManager
+from .config import Configs
+from .providers import StoreProvider, Providers
 from .stages import Stages
 
 class SecretProvider(StoreProvider):
@@ -13,17 +13,17 @@ class SecretProvider(StoreProvider):
     def get(self, stage, key):
         raise NotImplementedError()
 
-class SecretsClass():
+class SecretManager():
 
     def validate_key(self, key):
-        provider = ProviderManager.SecretProvider()
+        provider = Providers.SecretProvider()
         Logger.info("Start validating secret key...")
         return provider.validate_key(key)
 
     def list(self, stage, uninherited, decrypt):
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.SecretProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.SecretProvider()
         if uninherited:
             Logger.info(f"Start listing uninherited SSM secrets: project={project}, application={application}, stage={Stages.shorten(stage)}")
         else:
@@ -32,34 +32,34 @@ class SecretsClass():
 
     def add(self, stage, key, value):
         self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.SecretProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.SecretProvider()
         Logger.info(f"Start adding secrets: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
         return provider.add(project, application, stage, key, value)
 
     def get(self, stage, key, revision):
         # self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.SecretProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.SecretProvider()
         Logger.info(f"Start getting secret value: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
         return provider.get(project, application, stage, key, revision)
 
     def history(self, stage, key):
         # self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.SecretProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.SecretProvider()
         Logger.info(f"Start getting secret history: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
         return provider.history(project, application, stage, key)
 
     def delete(self, stage, key):
         # self.validate_key(key)
-        project = Config.project
-        application = Config.application
-        provider = ProviderManager.SecretProvider()
+        project = Configs.project
+        application = Configs.application
+        provider = Providers.SecretProvider()
         Logger.info(f"Start deleting secret: project={project}, application={application}, stage={Stages.shorten(stage)}, key={key}")
         return provider.delete(project, application, stage, key)
 
-Secrets = SecretsClass()
+Secrets = SecretManager()
